// Pyramidal_LucasKanade: Optical Flow
#include "opencv\cv.hpp"
#include <iostream>
#include <stdlib.h>

using namespace cv;
using namespace std;

#define MAX_CORNERS 500
#define ESCAPE 27

int main(int argc, char* argv[])
{

	VideoCapture capture;

	// Objects
	Mat frame;

	// keyboard pressed
	char keypressed = 0;
	bool success;

	// Load image from disk
	capture.open(0);
	// if not success, exit program
	if (!capture.isOpened()){
		cout << "error in VideoCapture: check path file" << endl;
		getchar();
		return 1;
	}

	/// Parameters for Shi-Tomasi algorithm
	vector<Point2f> cornersA, cornersB;
	double qualityLevel = 0.01;
	double minDistance = 10;
	int blockSize = 3;
	bool useHarrisDetector = false; // cornerMinEigenVal then
	double k = 0.04;
	int maxCorners = MAX_CORNERS;

	// winsize has to be 11 or 13, otherwise nothing is found
	vector<uchar> status;
	vector<float> error;
	int winsize = 11;
	int maxlvl = 5;

	// Objects
	Mat img_prev, img_next, grayA, grayB;
	
	success = capture.read(frame);
	// if no success exit program
	if (success == false){
		cout << "Cannot read the frame from file" << endl;
		getchar();
		return 1;
	}

	img_prev = frame.clone();

	// Windows for all the images
	namedWindow("Corners A", CV_WINDOW_AUTOSIZE);
	namedWindow("Corners B", CV_WINDOW_AUTOSIZE);

	while (keypressed != ESCAPE)
	{
		// read frame by frame in a loop
		success = capture.read(frame);
		// if no success exit program
		if (success == false){
			cout << "Cannot read the frame from file" << endl;
			return 1;
		}

		img_next = frame.clone();

		// convert to grayScale
		cvtColor(img_prev, grayA, CV_RGB2GRAY);
		cvtColor(img_next, grayB, CV_RGB2GRAY);

		/// Apply corner detection
		goodFeaturesToTrack(grayA,
			cornersA,
			maxCorners,
			qualityLevel,
			minDistance,
			Mat(),
			blockSize,
			useHarrisDetector,
			k);

		//cornerSubPix(grayA, cornersA, Size(15, 15), Size(-1, -1),
		//            TermCriteria(TermCriteria::COUNT + TermCriteria::EPS, 20, 0.03));

		calcOpticalFlowPyrLK(grayA, grayB, cornersA, cornersB, status, error,
			Size(winsize, winsize), maxlvl);

		/// Draw corners detected
		//cout << "Number of cornersA detected: " << cornersA.size() << endl;
		//cout << "Optical Flow corners detected: " << cornersB.size() << endl;
		for (int i = 0; i < cornersA.size(); i++)
		{
			line(img_prev, cornersA[i], cornersB[i], Scalar(0, 255, 0), 2);
			//circle(originalA, cornersA[i], radius, Scalar(255, 0, 0), -1, 8, 0);
			//circle(originalB, cornersB[i], radius, Scalar(0, 255, 0), -1, 8, 0);
		}


		// Show image in the name of the window
		imshow("Corners A", img_prev);
		imshow("Corners B", img_next);

		// Function for show the image in ms.
		keypressed = waitKey(1);
		img_prev = img_next;

	}
	// Free memory
	img_prev.release();
	img_next.release();
	grayA.release();
	grayB.release();
	destroyAllWindows();
	// End of the program
	return 0;
}


// Print the values on console
//void print_matrix_values(Mat src){
//	cout << "Matrix: " << endl;
//	for (int h = 0; h < src.cols * src.rows; h++)
//	{
//		cout << " " << (int)src.data[h];
//	}
//	cout << endl << endl;
//}
//
//


//
//int main() {
//	//Nombre de las imagenes que se van a cargar
//	char NombreImagen0[] = "image0.jpg";
//	char NombreImagen1[] = "image1.jpg";
//	char NombreOpticalFlow1[] = "OpticalFlow1.jpg";
//
//	IplImage* imgA = cvLoadImage(NombreImagen0,CV_LOAD_IMAGE_GRAYSCALE);
//	IplImage* imgB = cvLoadImage(NombreImagen1,CV_LOAD_IMAGE_GRAYSCALE);
//	CvSize img_sz = cvGetSize( imgA );
//	int win_size = 10;
//	IplImage* imgC = cvLoadImage(NombreOpticalFlow1,CV_LOAD_IMAGE_UNCHANGED);
//
//	// The first thing we need to do is get the features we want to track.
//	IplImage* eig_image = cvCreateImage( img_sz, IPL_DEPTH_32F, 1 );
//	IplImage* tmp_image = cvCreateImage( img_sz, IPL_DEPTH_32F, 1 );
//	int corner_count = MAX_CORNERS;
//	CvPoint2D32f* cornersA = new CvPoint2D32f[ MAX_CORNERS ];
//	cvGoodFeaturesToTrack(imgA,NULL,NULL,cornersA,&corner_count,0.01,5.0,0,3,0,0.04);
//	cvFindCornerSubPix(imgA,cornersA,corner_count,cvSize(win_size,win_size),cvSize(-1,-1),
//		cvTermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,20,0.03));
//	
//	// Call the Lucas Kanade algorithm
//	char features_found[ MAX_CORNERS ];
//	float feature_errors[ MAX_CORNERS ];
//	CvSize pyr_sz = cvSize( imgA->width+8, imgA->height/3 );
//	IplImage* pyrA = cvCreateImage( pyr_sz, IPL_DEPTH_32F, 1 );
//	IplImage* pyrB = cvCreateImage( pyr_sz, IPL_DEPTH_32F, 1 );
//	CvPoint2D32f* cornersB = new CvPoint2D32f[ MAX_CORNERS ];
//	cvCalcOpticalFlowPyrLK(imgA,imgB,pyrA,pyrB,cornersA,cornersB,corner_count,cvSize( win_size,win_size ),
//		5,features_found,feature_errors,cvTermCriteria( CV_TERMCRIT_ITER | CV_TERMCRIT_EPS, 20, .3 ),0);
//	
//	// Now make some image of what we are looking at:
//	for( int i=0; i<corner_count; i++ ) {
//		if( features_found[i]==0|| feature_errors[i]>550 ) {
//			printf("Error is %f\n",feature_errors[i]);
//			continue;
//		}
//		printf("Got it\n");
//		CvPoint p0 = cvPoint(cvRound( cornersA[i].x ),cvRound( cornersA[i].y ));
//		CvPoint p1 = cvPoint(cvRound( cornersB[i].x ),cvRound( cornersB[i].y ));
//		cvLine( imgC, p0, p1, CV_RGB(255,0,0),2 );
//	}
//
//	cvNamedWindow("ImageA",0);
//	cvNamedWindow("ImageB",0);
//	cvNamedWindow("LKpyr_OpticalFlow",0);
//	cvShowImage("ImageA",imgA);
//	cvShowImage("ImageB",imgB);
//	cvShowImage("LKpyr_OpticalFlow",imgC);
//	cvWaitKey(0);
//	return 0;
//}